// (c) alaah

import React    from 'react';
import ReactDOM from 'react-dom';
import './index.css';

const Square = props => {
    let className = "square";
    let disabled = props.value.uncovered || props.win;
    let style = {};
   
    if(props.win !== 1) {
        if(props.value.uncovered) {
            className += " uncovered";
        } else {
            className += " covered";
        }

        if(props.win === 2 && props.value.bad && !props.value.flagged) {
            style = {backgroundImage: 'url("' + props.url + '")'};
        }
        if(props.win === 2 && !props.value.bad && props.value.flagged) {
            className += " nun";
        } else if(props.value.flagged) {
            className += " cross";
        }
    } else {
        if(props.value.uncovered) {
            className += " uncovered-win";
        } else {
            className += " covered-win";
        }    
    }

    style.width      = props.cell;
    style.height     = props.cell;
//    style.lineHeight = props.cell;

    return (
        <button className={className}
                disabled={disabled}
                onClick={props.onClick}
                style={style}
                onContextMenu={e => {props.onContextMenu(); e.preventDefault();}}>
            {props.value.value > 0 ? props.value.value : null}
        </button>
    );
}
        
class Board extends React.Component {
    renderSquare(i, url) {
        return <td><Square cell={this.props.cell} url={url} win={this.props.win} value={this.props.squares[i]} onClick={() => this.props.onClick(i)} onContextMenu={() => this.props.onContextMenu(i)} /></td>;
    }

    render() {
        const s = [];
	const weebUrl = '/res/weeb/';
        const weebCount = 3514;
        for(let i = 0; i < this.props.height; ++i) {
            const items = [];
            for(let j = 0; j < this.props.width; ++j) {
                if(this.props.squares[i * this.props.width + j].bad && this.props.win === 2) {
                    items.push(this.renderSquare(i * this.props.width + j, weebUrl + parseInt(Math.random() * weebCount) + '.jpg'));
                } else {
                    items.push(this.renderSquare(i * this.props.width + j), '');
                }
            }
            s.push(<tr>{items}</tr>);
        }
        return (
            <table>
                {s}
            </table>
        );
    }
}

class Game extends React.Component {
    constructor(props) {
        super(props);
        let squares = new Array(this.props.width * this.props.height);
        for(let i = 0; i < this.props.width * this.props.height; ++i) {
            squares[i] = {    
                bad: false,
                flagged: false,
                uncovered: false,
                value: 0,
            };
        }
        for(let i = 0; i < this.props.animes; ++i) {
            let n;
            do {
                n = parseInt(Math.random() * this.props.width * this.props.height);
            } while(squares[n].bad);
            squares[n].bad = true;
        }
        this.state = { 
            squares: squares,
            win: 0,
            width: 0,
            height: 0,
        };
        this.count = this.props.width * this.props.height;
        this.updateWindowDimensions = this.updateWindowDimensions.bind(this);
    }

    componentDidMount() {
          this.updateWindowDimensions();
          window.addEventListener('resize', this.updateWindowDimensions);
    }

    componentWillUnmount() {
        window.removeEventListener('resize', this.updateWindowDimensions);
    }
    
    updateWindowDimensions() {
          this.setState({ 
            width:  window.innerWidth, 
            height: window.innerHeight, 
        });
    }

    clear(squares, x, y) {
        if(x < 0 || y < 0 || x >= this.props.width || y >= this.props.height) {
            return;
        }

        const i = y * this.props.width + x;

        if(!squares[i].flagged && !squares[i].bad && !squares[i].uncovered) {
            let count = 0;
            for(let j = -1; j <= 1; ++j) {
                for(let k = -1; k <= 1; ++k) {
                    if((j !== 0 || k !== 0) && y + k >= 0 && y + k < this.props.height && x + j >= 0 && x + j < this.props.width) {
                        if(squares[(y + k) * this.props.width + (x + j)].bad) {
                            count++;  
                        }
                    }
                }
            }
        
            squares[i].value = count;
            squares[i].uncovered = true;
            this.count--;
            
            if(count === 0) {
                this.clear(squares, x - 1, y - 1);
                this.clear(squares, x    , y - 1);
                this.clear(squares, x + 1, y - 1);
                this.clear(squares, x - 1, y    );
                this.clear(squares, x + 1, y    );
                this.clear(squares, x - 1, y + 1);
                this.clear(squares, x    , y + 1);
                this.clear(squares, x + 1, y + 1);
            }
        }
    }

    onLeftClick(i) {
        let squares = this.state.squares.slice();
            //alert(i + ' ' + i % this.props.width + ' ' + parseInt(i / this.props.width));
        
        if(squares[i].flagged) {
            // nada
        } else if(squares[i].bad) {
            if(this.count !== this.props.width * this.props.height) {
                this.setState({
                    win: 2,
                });
            }
        } else {
            this.clear(squares, i % this.props.width, parseInt(i / this.props.width), true);
            if(this.count === this.props.animes) {
                this.setState({
                    win: 1,
                });
            }
        }

        this.setState({
            squares: squares,
        });
    }

    onRightClick(i) {
        let squares = this.state.squares.slice();
        
        if(!squares[i].uncovered) {
            squares[i].flagged = !squares[i].flagged;
        }
        
        this.setState({
            squares: squares,
        });
    }

    render() {
        return (
            <div className="game">
            <div className={"board" + (this.state.win === 1 ? " jesus" : "")}>
                   <Board width={this.props.width} 
                          height={this.props.height}
                          cell={this.state.width / this.props.width > this.state.height / this.props.height ? this.state.height / this.props.height : this.state.width / this.props.width}
                          win={this.state.win} 
                          squares={this.state.squares} 
                          onClick={i => this.onLeftClick(i)} 
                          onContextMenu={i => this.onRightClick(i)} 
                />
            </div>
            </div>
        );
    }
}

class Form extends React.Component {
    constructor() {
        super();

        this.state = {
            width: 16,
            height: 16,
            animesPct: 270,
            animesTotal: parseInt(16 * 16 * 270 / 1000.),
            enabled: false,
        };
    }

    updateAnime() {
        this.setState({
            animesTotal: parseInt(this.state.width * this.state.height * this.state.animesPct / 1000.),
        });
    }

    render() {
        if(this.state.enabled) {
            return <Game width={this.state.width} height={this.state.height} animes={this.state.animesTotal}/>
        } else {
            return (
                <form className="config">
                    <div className="title">Weebsweeper</div>
                    <div className="desc">Current objective: reach The Lord and don't be led astray by the CCP's devilish cartoons.</div>
                    Width: {this.state.width}
                    <input type="range"  name="width"  min="8" max="48" defaultValue="16" onChange={event => {
                        this.setState({
                            width: event.target.value
                        }, this.updateAnime);
                    }} />
                    Height: {this.state.height}
                    <input type="range"  name="height" min="8" max="48" defaultValue="16" onChange={event => {
                       this.setState({
                           height: event.target.value,
                       }, this.updateAnime);
                    }} />
                    Devils: {this.state.animesTotal}
                    <input type="range"  name="animes" min="50"  max="500"  defaultValue="270" onChange={event => {
                       this.setState({
                           animesPct: event.target.value,
                       }, this.updateAnime);
                    }} />
                    <input type="button" name="accept" className="button" value="Deus vult" onClick={() => {
                        this.setState({
                            enabled: true,
                        });
                    }}/>
                </form>
            );
        }
    }
}

ReactDOM.render(
    <div className="main">
        <Form />
    </div>,
    document.getElementById('root')
);
